﻿using System.Collections.Generic;
using RimWorld;
using UnityEngine;
using Verse;

namespace Mincho_HPF
{
    // Token: 0x02000005 RID: 5
    public class CompResourceHarvestable : ThingComp
    {
        // Token: 0x17000002 RID: 2
        // (get) Token: 0x06000008 RID: 8 RVA: 0x000020E2 File Offset: 0x000002E2
        public CompProperties_ResourceHarvestable Props
        {
            get
            {
                return (CompProperties_ResourceHarvestable)this.props;
            }
        }

        // Token: 0x17000003 RID: 3
        // (get) Token: 0x06000009 RID: 9 RVA: 0x00002594 File Offset: 0x00000794
        protected bool Active
        {
            get
            {
                if (this.parent.Faction == null)
                {
                    return false;
                }
                Pawn pawn = this.parent as Pawn;
                using (List<Constraint>.Enumerator enumerator = this.Props.constraints.GetEnumerator())
                {
                    while (enumerator.MoveNext())
                    {
                        if (!enumerator.Current.CheckActiveCondition(this, pawn, null))
                        {
                            return false;
                        }
                    }
                }
                return true;
            }
        }

        // Token: 0x17000004 RID: 4
        // (get) Token: 0x0600000A RID: 10 RVA: 0x000020EF File Offset: 0x000002EF
        public bool ActiveAndFull
        {
            get
            {
                return this.Active && this.fullness >= 1f;
            }
        }

        // Token: 0x0600000B RID: 11 RVA: 0x0000210B File Offset: 0x0000030B
        public override void PostExposeData()
        {
            base.PostExposeData();
            Scribe_Values.Look<float>(ref this.fullness, this.Props.saveUniqueKey, 0f, false);
        }

        // Token: 0x0600000C RID: 12 RVA: 0x00002610 File Offset: 0x00000810
        public override void CompTick()
        {
            if (this.Active)
            {
                float num = 1f / (this.Props.intervalDays * 60000f);
                Pawn pawn = this.parent as Pawn;
                if (pawn != null)
                {
                    num *= PawnUtility.BodyResourceGrowthSpeed(pawn);
                }
                this.fullness += num;
                if (this.fullness > 1f)
                {
                    this.fullness = 1f;
                }
            }
        }

        // Token: 0x0600000D RID: 13 RVA: 0x0000267C File Offset: 0x0000087C
        public void Gathered(Pawn doer, StatDef stat)
        {
            if (!this.Active)
            {
                this.fullness = 0f;
            }
            if (!Rand.Chance(doer.GetStatValue(stat, true)))
            {
                MoteMaker.ThrowText((doer.DrawPos + this.parent.DrawPos) / 2f, this.parent.Map, "TextMote_ProductWasted".Translate(), 3.65f);
            }
            else
            {
                int i = GenMath.RoundRandom((float)this.Props.amount * this.fullness);
                while (i > 0)
                {
                    int num = Mathf.Clamp(i, 1, this.Props.thingDef.stackLimit);
                    i -= num;
                    Thing thing = ThingMaker.MakeThing(this.Props.thingDef, null);
                    thing.stackCount = num;
                    GenPlace.TryPlaceThing(thing, doer.Position, doer.Map, ThingPlaceMode.Near, null, null, default(Rot4));
                }
            }
            this.fullness = 0f;
        }

        // Token: 0x0600000E RID: 14 RVA: 0x00002770 File Offset: 0x00000970
        public override string CompInspectStringExtra()
        {
            if (!this.Active)
            {
                return null;
            }
            if (this.Props.inspectText == null)
            {
                return null;
            }
            //원래는 inspectText
            return this.Props.thingDef.label.Translate() + ": " + this.fullness.ToStringPercent();
        }

        // Token: 0x0400000A RID: 10
        private float fullness;
    }
}
