﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using Verse;
using Verse.AI;

namespace Mincho_HPF
{
	// Token: 0x0200001D RID: 29
	public class JobDriver_GatherPawnResources : JobDriver
	{
		// Token: 0x06000037 RID: 55 RVA: 0x00002749 File Offset: 0x00000949
		public override void ExposeData()
		{
			base.ExposeData();
			Scribe_Values.Look<float>(ref this.gatherProgress, "gatherProgress", 0f, false);
		}

		// Token: 0x06000038 RID: 56 RVA: 0x00002768 File Offset: 0x00000968
		public override bool TryMakePreToilReservations(bool errorOnFailed)
		{
			Pawn pawn = this.pawn;
			LocalTargetInfo target = this.job.GetTarget(TargetIndex.A);
			Job job = this.job;
			return pawn.Reserve(target, job, 1, -1, null, errorOnFailed);
		}

		// Token: 0x06000039 RID: 57 RVA: 0x0000279A File Offset: 0x0000099A
		protected override IEnumerable<Toil> MakeNewToils()
		{
			HPFJobDef def = this.job.def as HPFJobDef;
			if (def == null)
			{
				Log.Error("def is not HPFJobDef. please use HPF.HPFJobDef instead of JobDef.");
				yield break;
			}
			this.FailOnDespawnedNullOrForbidden(TargetIndex.A);
			this.FailOnNotCasualInterruptible(TargetIndex.A);
			yield return Toils_Goto.GotoThing(TargetIndex.A, PathEndMode.Touch);
			Toil wait = new Toil();
			wait.FailOnDespawnedOrNull(TargetIndex.A);
			wait.FailOnCannotTouch(TargetIndex.A, PathEndMode.Touch);
			wait.initAction = delegate ()
			{
				Pawn actor = wait.actor;
				Pawn pawn = (Pawn)this.job.GetTarget(TargetIndex.A).Thing;
				actor.pather.StopDead();
				PawnUtility.ForceWait(pawn, 15000, null, true);
			};
			wait.tickAction = delegate ()
			{
				HPFJobDef hpfjobDef = (HPFJobDef)this.job.def;
				Pawn actor = wait.actor;
				actor.skills.Learn(def.activeSkill, def.xpPerTick, false);
				this.gatherProgress += actor.GetStatValue(def.activeStat, true);
				if (this.gatherProgress >= hpfjobDef.totalWork)
				{
					ThingWithComps thingWithComps = this.job.GetTarget(TargetIndex.A).Thing as ThingWithComps;
					IEnumerable<CompResourceHarvestable> source = (thingWithComps != null) ? thingWithComps.GetComps<CompResourceHarvestable>() : null;
					Func<CompResourceHarvestable, bool> predicate = ((CompResourceHarvestable x) => x.Props.harvestJobDef == this.job.def);
					foreach (CompResourceHarvestable compResourceHarvestable in source.Where(predicate))
					{
						compResourceHarvestable.Gathered(this.pawn, hpfjobDef.activeStat);
					}
					actor.jobs.EndCurrentJob(JobCondition.Succeeded, true, true);
				}
			};
			wait.AddFinishAction(delegate
			{
				Pawn pawn = (Pawn)this.job.GetTarget(TargetIndex.A).Thing;
				if (pawn != null && pawn.CurJobDef == JobDefOf.Wait_MaintainPosture)
				{
					pawn.jobs.EndCurrentJob(JobCondition.InterruptForced, true, true);
				}
			});
			wait.AddEndCondition(delegate
			{
				ThingWithComps thingWithComps = this.job.GetTarget(TargetIndex.A).Thing as ThingWithComps;
				IEnumerable<CompResourceHarvestable> source = (thingWithComps != null) ? thingWithComps.GetComps<CompResourceHarvestable>() : null;
				Func<CompResourceHarvestable, bool> predicate = ((CompResourceHarvestable x) => x.Props.harvestJobDef == this.job.def);
				using (IEnumerator<CompResourceHarvestable> enumerator = source.Where(predicate).GetEnumerator())
				{
					while (enumerator.MoveNext())
					{
						if (!enumerator.Current.ActiveAndFull)
						{
							return JobCondition.Incompletable;
						}
					}
				}
				return JobCondition.Ongoing;
			});
			wait.defaultCompleteMode = ToilCompleteMode.Never;
			wait.WithProgressBar(TargetIndex.A, () => this.gatherProgress / ((HPFJobDef)this.job.def).totalWork, false, -0.5f, false);
			wait.activeSkill = (() => def.activeSkill);
			yield return wait;
			yield break;
		}

		// Token: 0x0400001F RID: 31
		private float gatherProgress;
	}
}
