﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using Verse;
using Verse.AI;

namespace Mincho_HPF
{
    public class JobDriver_GatherPawnSelfResources : JobDriver
    {
        private float gatherProgress;

        public override void ExposeData()
        {
            base.ExposeData();
            Scribe_Values.Look(ref gatherProgress, "gatherProgress", 0f);
        }

        public override bool TryMakePreToilReservations(bool errorOnFailed)
        {
            return true;
        }

        protected override IEnumerable<Toil> MakeNewToils()
        {
            HPFJobDef def = job.def as HPFJobDef;
            if (def == null)
            {
                Log.Error("def is not HPFJobDef. please use HPF.HPFJobDef instead of JobDef.");
                yield break;
            }
            Toil wait = new Toil();
            ToilEffects.WithProgressBar(wait, TargetIndex.A, (Func<float>)(() => gatherProgress / ((HPFJobDef)job.def).totalWork), false, -0.5f);
            ToilFailConditions.FailOnDespawnedOrNull(wait, TargetIndex.A);
            wait.defaultCompleteMode = ToilCompleteMode.Never;
            wait.activeSkill = () => def.activeSkill;
            wait.initAction = delegate
            {
                wait.actor.pather.StopDead();
            };
            wait.tickAction = delegate
            {
                Pawn actor = wait.actor;
                gatherProgress += StatExtension.GetStatValue(actor, def.activeStat);
                if (gatherProgress >= ((HPFJobDef)job.def).totalWork)
                {
                    foreach (CompResourceHarvestable item in from x in actor.GetComps<CompResourceHarvestable>()
                                                             where x.Props.harvestJobDef == job.def
                                                             select x)
                    {
                        item.Gathered(actor, def.activeStat);
                    }
                    actor.jobs.EndCurrentJob(JobCondition.Succeeded);
                }
            };
            yield return wait;
        }
    }

}
