﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using Verse;
using Verse.AI;

namespace Mincho_HPF
{
	public class WorkGiver_GatherPawnResources : WorkGiver_Scanner
	{
		public override IEnumerable<Thing> PotentialWorkThingsGlobal(Pawn pawn)
		{
			foreach (Pawn pawn2 in pawn.Map.mapPawns.FreeColonistsAndPrisonersSpawned)
			{
				if (pawn2.GetComp<CompResourceHarvestable>() != null)
				{
					yield return pawn2;
				}
			}
			yield break;
		}
		public override PathEndMode PathEndMode
		{
			get
			{
				HPFWorkGiverDef hpfworkGiverDef = this.def as HPFWorkGiverDef;
				if (hpfworkGiverDef == null)
				{
					return 0;
				}
				HPFJobDef hpfjobDef = hpfworkGiverDef.harvestJobDef as HPFJobDef;
				if (hpfjobDef == null)
				{
					return 0;
				}
				if (hpfjobDef.isSelf)
				{
					return 0;
				}
				return (PathEndMode)2;
			}
		}

		public override bool HasJobOnThing(Pawn pawn, Thing thing, bool forced = false)
		{
			Pawn pawn2 = thing as Pawn;
			if (pawn2 == null)
			{
				return false;
			}
			HPFWorkGiverDef hpfworkGiverDef = this.def as HPFWorkGiverDef;
			if (hpfworkGiverDef == null)
			{
				Log.Error("def is not HPFWorkGiverDef. please use HPF.HPFWorkGiverDef instead of WorkGiverDef.");
				return false;
			}
			HPFJobDef jobDef = hpfworkGiverDef.harvestJobDef as HPFJobDef;
			if (jobDef == null)
			{
				Log.Error("job def is not HPFJobDef.");
				return false;
			}
			if (!jobDef.isSelf && pawn == thing)
			{
				return false;
			}
			if (!jobDef.isSelf)
			{
				IEnumerable<CompResourceHarvestable> comps = pawn2.GetComps<CompResourceHarvestable>();
				Func<CompResourceHarvestable, bool> predicate = default;
				if (predicate  == null)
				{
					predicate = (CompResourceHarvestable x) => x.Props.harvestJobDef == jobDef && x.ActiveAndFull;
				}
				foreach (CompResourceHarvestable compResourceHarvestable in comps.Where(predicate))
				{
					if (PawnUtility.CanCasuallyInteractNow(pawn2, false, false, false) && ReservationUtility.CanReserve(pawn, pawn2, 1, -1, null, false))
					{
						return true;
					}
				}
				return false;
			}
			if (pawn == thing && pawn.GetComps<CompResourceHarvestable>().Any((CompResourceHarvestable x) => x.Props.harvestJobDef == jobDef && x.ActiveAndFull))
			{
				return true;
			}
			return false;
		}

		public override Job JobOnThing(Pawn pawn, Thing t, bool forced = false)
		{
			HPFWorkGiverDef hpfworkGiverDef = this.def as HPFWorkGiverDef;
			if (hpfworkGiverDef == null)
			{
				Log.Error("def is not HPFWorkGiverDef. please use HPF.HPFWorkGiverDef instead of WorkGiverDef.");
				return null;
			}
			return new Job(hpfworkGiverDef.harvestJobDef, t);
		}
	}
}
