﻿using HarmonyLib;
using RimWorld;
using System;
using System.Linq;
using Verse;
using static WearSetting.WearSettingMethod;

namespace WearSetting
{
    [StaticConstructorOnStartup]
    public static class Harmony_EquipmentUtility
    {
        static Harmony_EquipmentUtility()
        {
            var harmony = new Harmony("Mincho_The_Mint_Choco_Slime");

            harmony.Patch(AccessTools.Method(typeof(EquipmentUtility), "CanEquip", new Type[]
                {
                typeof(Thing),
                typeof(Pawn),
                typeof(string).MakeByRefType(),
                typeof(bool)
                }, null), null, new HarmonyMethod(patchType, "CanEquipPostfix", null), null, null);
        }

        public static void CanEquipPostfix(ref bool __result, Thing thing, Pawn pawn, ref string cantReason)
        {
            if (__result)
            {

                foreach (var (wearSetting, wearList) in from wearSettingDef in DefDatabase<WearSettingDef>.AllDefs
                                                        where pawn.def == wearSettingDef.targetRace
                                                        from wearSetting in wearSettingDef.wearSetting
                                                        from wearList in wearSetting.wearList
                                                        select (wearSetting, wearList))
                {
                    foreach (var equipment in pawn.equipment.AllEquipmentListForReading)
                    {
                        if (equipment.def == wearList && wearSetting.lockWeapon)
                        {
                            __result = false;
                            if (wearSetting.lockWeaponCantReason == null)
                            {
                                cantReason = string.Format("LockWeaponCantReason_Default".Translate(
                                    pawn.def.LabelCap,
                                    pawn.LabelShort,
                                    thing.def.LabelCap
                                    ));
                            }
                            else
                            {
                                cantReason = string.Format(wearSetting.lockWeaponCantReason.Translate(
                                    pawn.def.LabelCap,
                                    pawn.LabelShort,
                                    thing.def.LabelCap
                                    ));
                            }
                            return;
                        }

                    }
                }

                foreach (var (wearSetting, wearList) in from wearSettingDef in DefDatabase<WearSettingDef>.AllDefs
                                                        where pawn.def == wearSettingDef.targetRace
                                                        from wearSetting in wearSettingDef.wearSetting
                                                        from wearList in wearSetting.wearList
                                                        where (thing.def.IsApparel || thing.def.IsWeapon) && thing.def == wearList
                                                        select (wearSetting, wearList))
                {

                    WearSettingBoolMaker(wearSetting, pawn, out Condition condition, out bool age, out bool gender, out string genderString, out bool bodyType, out string bodyTypeString, out bool orHediff, out string orHediffString, out bool andHediff, out string andHediffString, out bool forbiddenHediff, out string forbiddenHediffString);



                    if (!age || !bodyType || !gender || !orHediff || !andHediff || forbiddenHediff)
                    {
                        BoolString boolString = wearSetting.boolString;

                        BoolString(boolString.ageFalse.Translate(), boolString.ageTrue.Translate(), age, out string ageBoolString);
                        BoolString(boolString.genderFalse.Translate(), boolString.genderTrue.Translate(), gender, out string genderBoolString);
                        BoolString(boolString.bodyTypeFalse.Translate(), boolString.bodyTypeTrue.Translate(), bodyType, out string bodyTypeBoolString);
                        BoolString(boolString.orHediffFalse.Translate(), boolString.orHediffTrue.Translate(), orHediff, out string orHediffBoolString);
                        BoolString(boolString.andHediffFalse.Translate(), boolString.andHediffTrue.Translate(), andHediff, out string andHediffBoolString);
                        BoolString(boolString.forbiddenHediffFalse.Translate(), boolString.forbiddenHediffTrue.Translate(), !forbiddenHediff, out string forbiddenHediffBoolString);

                        __result = false;
                        if (wearSetting.cantReason == null)
                        {
                            cantReason = string.Format("WearSettingForbidden".Translate(
                                pawn.def.LabelCap,
                                pawn.LabelShort,
                                thing.def.LabelCap,

                                ageBoolString,
                                condition.age.min,
                                condition.age.max,

                                genderBoolString,
                                genderString,

                                bodyTypeBoolString,
                                bodyTypeString,

                                orHediffBoolString,
                                orHediffString,

                                andHediffBoolString,
                                andHediffString,

                                forbiddenHediffBoolString,
                                forbiddenHediffString
                                ));
                        }
                        else
                        {
                            cantReason = string.Format(wearSetting.cantReason.Translate(
                              pawn.def.LabelCap,
                              pawn.LabelShort,
                              thing.def.LabelCap,

                              ageBoolString,
                              condition.age.min,
                              condition.age.max,

                              genderBoolString,
                              genderString,

                              bodyTypeBoolString,
                              bodyTypeString,

                              orHediffBoolString,
                              orHediffString,

                              andHediffBoolString,
                              andHediffString,

                              forbiddenHediffBoolString,
                              forbiddenHediffString
                              ));
                        }
                        return;
                    }
                }
            }
        }


        private static readonly Type patchType = typeof(Harmony_EquipmentUtility);
    }
}
