﻿using RimWorld;
using System.Linq;
using Verse;

namespace WearSetting
{
    [StaticConstructorOnStartup]
    //대상 종족에 콤프 추가
    public class AddComp
    {
        static AddComp()
        {
            foreach (var wearSetting in DefDatabase<WearSettingDef>.AllDefs)
            {
                wearSetting.targetRace.comps.Add(new CompProperties(typeof(ThingComp_WearSetting)));
            }
        }
    }
    public class ThingComp_WearSetting : ThingComp
    {
        public override void CompTickRare()
        {
            if (pawn != null && pawn.Spawned)
            {
                base.CompTickRare();

                string addHediffString = "";
                string partsToAffectString = "";

                Pawn_ApparelTracker apparelTracker = pawn.apparel;
                Pawn_EquipmentTracker equipmentTracker = pawn.equipment;
                foreach (var (wearSetting, wearList) in from wearSettingDef in DefDatabase<WearSettingDef>.AllDefs
                                                        from wearSetting in wearSettingDef.wearSetting
                                                        from wearList in wearSetting.wearList
                                                        select (wearSetting, wearList))
                {
                    //의복
                    if (wearList.IsApparel)
                    {
                        for (int n = 0; n < apparelTracker.WornApparel.Count; n++)
                        {
                            Apparel wornApparel = apparelTracker.WornApparel[n];
                            if (wearList == wornApparel.def)
                            {
                                WearSettingMethod.WearSettingBoolMaker(wearSetting, pawn, out Condition condition, out bool age, out bool gender, out string genderString, out bool bodyType, out string bodyTypeString, out bool orHediff, out string orHediffString, out bool andHediff, out string andHediffString, out bool forbiddenHediff, out string forbiddenHediffString);
                                bool forceDropHediffBool = ForceDropHediffBool(wearSetting, pawn);

                                if (forceDropHediffBool || !age || !bodyType || !gender || !orHediff || !andHediff || forbiddenHediff)
                                {
                                    if (wearSetting.forceDrop || forceDropHediffBool)
                                    {
                                        apparelTracker.TryDrop(wornApparel, out Apparel apparelTemp, pawn.Position, true);
                                        ForceDropMessage(wearSetting, wornApparel, condition, genderString, bodyTypeString, orHediffString, andHediffString, forbiddenHediffString);
                                        continue;
                                    }
                                }

                                else if (age && bodyType && gender && orHediff && andHediff && !forbiddenHediff)
                                {
                                    AddHediff(ref addHediffString, ref partsToAffectString, wearSetting, wornApparel, condition, genderString, bodyTypeString, orHediffString, andHediffString, forbiddenHediffString);
                                    LockApparel(apparelTracker, wearSetting, wornApparel, condition, genderString, bodyTypeString, orHediffString, andHediffString, forbiddenHediffString);
                                }
                            }
                        }
                    }
                    //무기
                    if (wearList.IsWeapon)
                    {
                        foreach (var allEquipment in equipmentTracker.AllEquipmentListForReading)
                        {
                            if (wearList == allEquipment.def)
                            {
                                WearSettingMethod.WearSettingBoolMaker(wearSetting, pawn, out Condition condition, out bool age, out bool gender, out string genderString, out bool bodyType, out string bodyTypeString, out bool orHediff, out string orHediffString, out bool andHediff, out string andHediffString, out bool forbiddenHediff, out string forbiddenHediffString);
                                bool forceDropHediffBool = ForceDropHediffBool(wearSetting, pawn);

                                if (forceDropHediffBool || !age || !bodyType || !gender || !orHediff || !andHediff || forbiddenHediff)
                                {
                                    if (wearSetting.forceDrop || forceDropHediffBool)
                                    {
                                        equipmentTracker.TryDropEquipment((ThingWithComps)allEquipment, out ThingWithComps thingWithCompsTemp, pawn.Position, true);
                                        ForceDropMessage(wearSetting, allEquipment, condition, genderString, bodyTypeString, orHediffString, andHediffString, forbiddenHediffString);
                                        continue;
                                    }
                                }
                                else if (age && bodyType && gender && orHediff && andHediff && !forbiddenHediff)
                                {
                                    AddHediff(ref addHediffString, ref partsToAffectString, wearSetting, allEquipment, condition, genderString, bodyTypeString, orHediffString, andHediffString, forbiddenHediffString);
                                }
                            }
                        }
                    }
                }
            }
        }

        private void LockApparel(Pawn_ApparelTracker apparel, WearSetting wearSetting, Apparel wornApparel, Condition condition, string genderString, string bodyTypeString, string orHediffString, string andHediffString, string forbiddenHediffString)
        {
            if (wearSetting.lockApparel)
            {
                bool tryLock = true;
                for (int i = 0; i < apparel.LockedApparel.Count; i++)
                {
                    if (apparel.LockedApparel[i] == wornApparel)
                    {
                        tryLock = false;
                        break;
                    }
                }
                if (tryLock)
                {
                    apparel.Lock(wornApparel);
                    if (wearSetting.lockApparelMessage != null)
                    {
                        Messages.Message(wearSetting.lockApparelMessage.Translate(
                            pawn.def.LabelCap,
                            pawn.LabelShort,
                            wornApparel.def.LabelCap,
                            condition.age.min,
                            condition.age.max,
                            genderString,
                            bodyTypeString,
                            orHediffString,
                            andHediffString,
                            forbiddenHediffString
                            ).CapitalizeFirst(), pawn, MessageTypeDefOf.NeutralEvent, true);
                    }
                }
            }
        }

        private void AddHediff(ref string addHediffString, ref string partsToAffectString, WearSetting wearSetting, Thing thing, Condition condition, string genderString, string bodyTypeString, string orHediffString, string andHediffString, string forbiddenHediffString)
        {
            HediffDef hediffDef = new HediffDef();
            for (int i = 0; i < wearSetting.addHediff.Count; i++)
            {
                addHediffString = "";
                partsToAffectString = "";
                AddHediff addHediff = wearSetting.addHediff[i];
                for (int k = 0; k < addHediff.hediff.Count; k++)
                {
                    hediffDef = addHediff.hediff[k];
                    if (!pawn.health.hediffSet.HasHediff(hediffDef, false))
                    {
                        if (HediffGiverUtility.TryApply(pawn, hediffDef, addHediff.partsToAffect))
                        {
                            if (k == addHediff.hediff.Count - 1)
                            {
                                addHediffString += hediffDef.LabelCap;
                            }
                            else
                                addHediffString += hediffDef.LabelCap + ", ";
                        }
                    }
                }
                for (int k = 0; k < addHediff.partsToAffect.Count; k++)
                {
                    if (k == addHediff.partsToAffect.Count - 1)
                    {
                        partsToAffectString += addHediff.partsToAffect[k].LabelCap;
                    }
                    else
                        partsToAffectString += addHediff.partsToAffect[k].LabelCap + ", ";
                }
                if (addHediff.addHediffMessage != null && !addHediffString.Equals(""))
                {
                    Messages.Message(addHediff.addHediffMessage.Translate(
                        pawn.def.LabelCap,
                        pawn.LabelShort,
                        thing.def.LabelCap,
                        condition.age.min,
                        condition.age.max,
                        genderString,
                        bodyTypeString,
                        orHediffString,
                        andHediffString,
                        forbiddenHediffString,
                        addHediffString,
                        partsToAffectString
                        ).CapitalizeFirst(), pawn, MessageTypeDefOf.NeutralEvent, true);
                }
            }
        }

        private void ForceDropMessage(WearSetting wearSetting, Thing dropThing, Condition condition, string genderString, string bodyTypeString, string orHediffString, string andHediffString, string forbiddenHediffString)
        {
            if (wearSetting.forceDropMessage != null)
            {
                Messages.Message(wearSetting.forceDropMessage.Translate(
                    pawn.def.LabelCap,
                    pawn.LabelShort,
                    dropThing.def.LabelCap,
                    condition.age.min,
                    condition.age.max,
                    genderString,
                    bodyTypeString,
                    orHediffString,
                    andHediffString,
                    forbiddenHediffString
                    ).CapitalizeFirst(), pawn, MessageTypeDefOf.NeutralEvent, true);
            }
        }

        public static bool ForceDropHediffBool(WearSetting wearSetting, Pawn pawn)
        {
            bool forceDropHediffBool = false;
            foreach (var forceDropHediff in wearSetting.forceDropHediff)
            {
                if (pawn.health.hediffSet.HasHediff(forceDropHediff, false))
                {
                    forceDropHediffBool = true;
                    break;
                }
            }

            return forceDropHediffBool;
        }

        public override void PostSpawnSetup(bool respawningAfterLoad)
        {
            base.PostSpawnSetup(respawningAfterLoad);
            this.pawn = (this.parent as Pawn);
        }
        public Pawn pawn { get; private set; }
    }
}
